HotCandy = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "CandyBar-2.0")

local HotCandy = HotCandy

local VERSION = tonumber(("$Revision: 61575 $"):match("%d+"))
HotCandy.revision = VERSION
HotCandy.versionstring = "1.0 |cffff8888r%d|r"
HotCandy.version = HotCandy.versionstring:format(VERSION)
HotCandy.date = ("$Date: 2008-02-16 06:45:10 -0500 (Sat, 16 Feb 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")

local BS = AceLibrary("Babble-Spell-2.2")
local media = AceLibrary:HasInstance("SharedMedia-1.0") and AceLibrary("SharedMedia-1.0") or { Fetch = function() return "Interface\\Addons\\HotCandy\\textures\\default" end, List = function() return {"BantoBar"} end, Register = function() end, }

local mType = media.MediaType and media.MediaType.STATUSBAR or "statusbar"

local itemSets = {
	["Stormrage"] = { 16897, 16898, 16899, 16900, 16901, 16902, 16903, 16904 },
	["Nordrassil"] = { 30216, 30217, 30219, 30220, 30221 },
	["Transcendence"] = { 16919, 16920, 16921, 16922, 16923, 16924, 16925, 16926 },
}
	-- spell = duration
local durations = {
	['Rejuvenation'] = 12,
	['Regrowth'] = 21,
	['Renew'] = 15,
	['Greater Heal'] = 0,
	['Lifebloom'] = 7,
	['Gift of the Naaru'] = 15,
}

local spell = {}
local track = {}
local noname, stackfirst

function HotCandy:OnInitialize()
	self:RegisterDB("HotCandyDB")
	self:RegisterDefaults("profile", {
		growup = false,
		texture = "BantoBar",
		barwidth = 200,
		barscale = 1,
		stackfirst = true,
		noname = false,
	})
	
	media:Register(mType, "BantoBar", "Interface\\Addons\\HotCandy\\textures\\default")
	
	self.options = {
		type = "group",
		args = {
			anchor = {
				name = "Show Anchor",
				desc = "Show the Anchor for moving the Bars",
				type = "execute",
				func = "ShowAnchors",
			},
			growup = {
				name = "Grow Up",
				desc = "Toggle Grow Up of the Bars",
				type = "toggle",
				get = function() return self.db.profile.growup end,
				set = function(v) self.db.profile.growup = v end,
			},
			texture = {
				type = "text",
				name = "Texture",
				desc = "Set the texture for the bars.",
				validate = media:List(mType),
				get = function() return self.db.profile.texture end,
				set = function(tex) self.db.profile.texture = tex end,
			},
			width = {
				type = "range",
				name = "Bar Width",
				desc = "Set the width of the bars.",
				min = 50, max = 500, step = 1, bigStep = 10,
				get = function() return self.db.profile.barwidth end,
				set = function(width) self.db.profile.barwidth = width end,
			},
			scale = {
				type = "range",
				name = "Bar Scale",
				desc = "Set the scale of the bars.",
				min = .1, max = 2, step = 0.01, bigStep = 0.05,
				isPercent = true,
				get = function() return self.db.profile.barscale end,
				set = function(scale) self.db.profile.barscale = scale end,
			},
			format = {
				type = "group",
				name = "Text Format",
				desc = "Configure the format of the text on the timer bar.",
				args = {
					noname = {
						type = "toggle",
						name = "No HoT name",
						desc = "Do not show the name of the HoT.",
						get = function() return self.db.profile.noname end,
						set = function(state) 
							self.db.profile.noname = state
							noname = state
						end,
					},
					stackfirst = {
						type = "toggle",
						name = "Stack First",
						desc = "Show HoT stack count before the name.",
						get = function() return self.db.profile.stackfirst end,
						set = function(pos) 
							self.db.profile.stackfirst = pos
							stackfirst = pos
						end,
					},
				},
			},
		}
	}
	
	self:RegisterChatCommand({ "/hot", "/hotcandy" }, self.options )
end

function HotCandy:OnEnable()
	self:SetupFrames()
	
	noname, stackfirst = self.db.profile.noname, self.db.profile.stackfirst
	
	self:RegisterEvent("UNIT_SPELLCAST_SENT", "SpellCastEvent")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "SpellCastEvent")
	self:RegisterBucketEvent("UNIT_INVENTORY_CHANGED", 1, "equipChanged")
	
	self:UpdateSetBonus()
end

function HotCandy:UpdateSetBonus()
	local _, class = UnitClass("player")
	
	for k,v in pairs(durations) do
		local key = BS[k]
		if not track[key] then
			track[key] = { icon = BS:GetSpellIcon(k) }
		end
		track[key].duration = v
	end
	
	if class == "DRUID" then
		if (self:getWornSetPieces("Stormrage") >= 8) then
			track[BS['Rejuvenation']].duration = track[BS['Rejuvenation']].duration + 3
		end
		if (self:getWornSetPieces("Nordrassil") >= 2) then
			track[BS['Regrowth']].duration = track[BS['Regrowth']].duration + 6
		end
	elseif class == "PRIEST" then
		if (self:getWornSetPieces("Transcendence") >= 8) then
			track[BS['Greater Heal']].duration = track[BS['Greater Heal']].duration + 15
		end
	end
end

-- taken from Threat-1.0
function HotCandy:getWornSetPieces(name)
	local ct = 0
	local data = itemSets[name]
	if data then
		for _, itemID in ipairs(data) do
			if IsEquippedItem( itemID ) then
				ct = ct + 1
			end
		end
	end
	return ct
end

function HotCandy:equipChanged(units)
	if (units and not units.player) or InCombatLockdown() then
		return
	end
	self:UpdateSetBonus()
end

function HotCandy:SpellCastEvent(unit, spellName, rank, target)
	if ( event == "UNIT_SPELLCAST_SENT" ) then
		spell['spell'] = spellName
		spell['target'] = target
	elseif ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then
		if spell['spell'] == spellName then
			self:FireSpell(spellName, spell['target'])
		end
		spell['spell'] = nil
	end
end

local fmt = string.format

local lb = BS['Lifebloom']
function HotCandy:FireSpell(spellName, spellTarget)
	local spell = track[spellName]
	if spell and spell.duration > 0 then
		local id = spellName..spellTarget
		local text = noname and nil or spellName
		if spellName == lb then
			local i = 1
			local buff, _, _, count, duration = UnitBuff(spellTarget, i)
			while buff and duration and buff ~= spellName do
				i = i + 1
				buff, _, _, count, duration = UnitBuff(spellTarget, i)
			end
			if count and count > 0 then
				count = min(count + 1, 3)
				if noname then
					text = tostring(count)
				elseif stackfirst then
					text = fmt("(%d)%s", count, text)
				else
					text = fmt("%s(%d)", text, count)
				end
			end
		end
		if text then
			text = text .. " - " .. spellTarget
		else
			text = spellTarget
		end
		self:ShowCandyBar(id, text, spell.duration, spell.icon, "green", "yellow", "orange", "red")
	end
end

function HotCandy:ShowCandyBar(id, text, time, icon, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10)
	id = "HotCandy"..id
	local u = self.db.profile.growup
	self:RegisterCandyBarGroup("HotCandyGroup")
	self:SetCandyBarGroupPoint("HotCandyGroup", u and "BOTTOM" or "TOP", self.frames.anchor, u and "TOP" or "BOTTOM", 0, 0)
	self:SetCandyBarGroupGrowth("HotCandyGroup", u)
	
	self:RegisterCandyBar(id, time, text, icon, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10)
	self:RegisterCandyBarWithGroup(id, "HotCandyGroup")
	self:SetCandyBarTexture(id, media:Fetch(mType, self.db.profile.texture))
	self:SetCandyBarFade(id, 5)
  self:SetCandyBarWidth(id, self.db.profile.barwidth)
	self:SetCandyBarScale(id, self.db.profile.barscale)
	self:StartCandyBar(id, true)
end
